import 'dart:async';
import 'dart:io';

import 'package:camera/camera.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/painting.dart';

import 'my_mask.dart';

enum MaskType { idCard, passport }

// A widget that allows users to take a picture using a given camera.
class IDCameraWidget extends StatefulWidget {
  final CameraDescription camera;
  final String title;
  final String firstDescription;
  final String secondDescription;
  final String questionToConfirmUseByTakenPhoto;
  final String rejectText;
  final String confirmText;
  final MaskType maskType;
  final ValueChanged<XFile?> onResult;

  const IDCameraWidget({
    Key? key,
    required this.camera,
    this.maskType = MaskType.idCard,
    required this.onResult,
    this.title = 'This is title',
    this.firstDescription = 'This is first description',
    this.secondDescription = 'This is second description',
    this.questionToConfirmUseByTakenPhoto =
        'This is question to confirm use by taken photo',
    this.rejectText = 'Captures',
    this.confirmText = 'Using this photo',
  }) : super(key: key);

  @override
  IDCameraWidgetState createState() => IDCameraWidgetState();
}

class IDCameraWidgetState extends State<IDCameraWidget> {
  late CameraController _controller;
  late Future<void> _initializeControllerFuture;
  late MaskType _maskType;
  String _imagePath = '';
  XFile? _xFileResult;
  bool _isShowIDCardMask = true;

  @override
  void initState() {
    super.initState();

    _maskType = widget.maskType;
    _isShowIDCardMask = _maskType == MaskType.idCard ? true : false;
    // To display the current output from the Camera,
    // create a CameraController.
    _controller = CameraController(
      // Get a specific camera from the list of available cameras.
      widget.camera,
      // Define the resolution to use.
      ResolutionPreset.max,
    );

    // Next, initialize the controller. This returns a Future.
    _initializeControllerFuture = _controller.initialize();
  }

  @override
  void dispose() {
    // Dispose of the controller when the widget is disposed.
    _controller.dispose();
    _closeStream();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    Size _size = MediaQuery.of(context).size;
    double _paddingTop = MediaQuery.of(context).padding.top;
    return Scaffold(
      // You must wait until the controller is initialized before displaying the
      // camera preview. Use a FutureBuilder to display a loading spinner until the
      // controller has finished initializing.
      body: Stack(
        fit: StackFit.expand,
        children: [
          FutureBuilder<void>(
            future: _initializeControllerFuture,
            builder: (context, snapshot) {
              if (snapshot.connectionState == ConnectionState.done) {
                // If the Future is complete, display the preview.
                return CameraPreview(_controller);
                // return CircularProgressIndicator();
              } else {
                // Otherwise, display a loading indicator.
                return const Center(child: CircularProgressIndicator());
              }
            },
          ),
          StreamBuilder<String>(
              stream: _cameraStream,
              builder: (context, snapshot) {
                if (snapshot.hasData && snapshot.data!.trim().isNotEmpty) {
                  _imagePath = snapshot.data!;
                } else {
                  _imagePath = '';
                }
                return _imagePath.isNotEmpty
                    ? Container(
                        constraints: const BoxConstraints.expand(),
                        child: Image.file(
                          File(_imagePath),
                        ),
                      )
                    : Container();
              }),
          Container(
            constraints: const BoxConstraints.expand(),
            child: Container(
              color: Colors.transparent,
              width: _size.width,
              height: _size.height,
              child: Stack(
                fit: StackFit.expand,
                children: [
                  StreamBuilder<String>(
                      stream: _cameraStream,
                      builder: (context, snapshot) {
                        final _backgroundColor =
                            snapshot.hasData && snapshot.data!.isNotEmpty
                                ? Colors.black
                                : Colors.black54;
                        return CustomPaint(
                          painter: MyMask(
                            outerBackgroundColor: _backgroundColor,
                            pathBorderType: _maskType,
                          ),
                        );
                      }),
                  Positioned(
                    top: 16 + _paddingTop,
                    left: 0,
                    right: 0,
                    child: Row(
                      mainAxisSize: MainAxisSize.max,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        Text(
                          widget.title,
                          style: const TextStyle(
                            fontSize: 14,
                            fontWeight: FontWeight.w500,
                            color: Colors.white,
                          ),
                        ),
                      ],
                    ),
                  ),
                  Positioned(
                    top: 8 + _paddingTop,
                    left: 16,
                    child: IconButton(
                      onPressed: () => Navigator.of(context).pop(),
                      icon: const Icon(
                        Icons.clear,
                        size: 30,
                      ),
                    ),
                  ),
                  Positioned(
                    top: (_size.height) / 2 -
                        _size.width *
                            0.9 /
                            3 *
                            (_isShowIDCardMask ? 2 : 4) /
                            2 -
                        30,
                    left: 16,
                    right: 16,
                    child: Row(
                      mainAxisSize: MainAxisSize.max,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        Text(
                          widget.firstDescription,
                          textAlign: TextAlign.center,
                          style: const TextStyle(
                            fontWeight: FontWeight.w400,
                            fontSize: 16,
                          ),
                        ),
                      ],
                    ),
                  ),
                  Positioned(
                    top: (_size.height) / 2 +
                        _size.width *
                            0.9 /
                            3 *
                            (_isShowIDCardMask ? 2 : 4) /
                            2 +
                        12,
                    left: 16,
                    right: 16,
                    child: Row(
                      mainAxisSize: MainAxisSize.min,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        Text(
                          widget.secondDescription,
                          textAlign: TextAlign.center,
                          style: const TextStyle(
                            fontWeight: FontWeight.w400,
                            fontSize: 12,
                            color: Color(0xffC7C7CC),
                          ),
                        ),
                      ],
                    ),
                  ),
                  StreamBuilder<String>(
                      stream: _cameraStream,
                      builder: (context, snapshot) {
                        if (snapshot.hasData && snapshot.data!.isNotEmpty) {
                          return Positioned(
                            left: 0,
                            right: 0,
                            bottom: 40,
                            child: Column(
                              children: [
                                Text(
                                  widget.questionToConfirmUseByTakenPhoto,
                                  style: const TextStyle(
                                      fontSize: 12,
                                      fontWeight: FontWeight.w400,
                                      color: Color(0xffC7C7CC)),
                                ),
                                const SizedBox(height: 16),
                                Row(
                                  children: [
                                    const SizedBox(width: 24),
                                    Expanded(
                                      child: GestureDetector(
                                        onTap: () {
                                          _cameraSink.add('');
                                        },
                                        child: Container(
                                          padding: const EdgeInsets.symmetric(
                                              vertical: 12),
                                          alignment: Alignment.center,
                                          decoration: BoxDecoration(
                                            color:
                                                Colors.white.withOpacity(0.1),
                                            borderRadius:
                                                BorderRadius.circular(8),
                                          ),
                                          child: Text(
                                            widget.rejectText,
                                            style: const TextStyle(
                                              fontWeight: FontWeight.w400,
                                              fontSize: 18,
                                            ),
                                          ),
                                        ),
                                      ),
                                    ),
                                    const SizedBox(width: 24),
                                    Expanded(
                                      child: GestureDetector(
                                        onTap: () {
                                          widget.onResult(_xFileResult);
                                          Navigator.of(context).pop();
                                        },
                                        child: Container(
                                          padding: const EdgeInsets.symmetric(
                                              vertical: 12),
                                          alignment: Alignment.center,
                                          decoration: BoxDecoration(
                                            color: Colors.white,
                                            borderRadius:
                                                BorderRadius.circular(8),
                                          ),
                                          child: Text(
                                            widget.confirmText,
                                            style: const TextStyle(
                                              fontWeight: FontWeight.w400,
                                              fontSize: 18,
                                              color: Colors.black,
                                            ),
                                          ),
                                        ),
                                      ),
                                    ),
                                    const SizedBox(width: 24),
                                  ],
                                ),
                              ],
                            ),
                          );
                        }

                        return Positioned(
                          left: 16,
                          right: 16,
                          bottom: 40,
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              GestureDetector(
                                onTap: () => _onCaptureImage(),
                                child: Container(
                                  padding: const EdgeInsets.all(20),
                                  decoration: const BoxDecoration(
                                    color: Color(0xffFFFFFF),
                                    shape: BoxShape.circle,
                                  ),
                                  child: const Icon(
                                    Icons.camera_alt,
                                    size: 32,
                                    color: Color(0xff3A3A3C),
                                  ),
                                ),
                              ),
                            ],
                          ),
                        );
                      }),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }

  void _onCaptureImage() async {
    // Take the Picture in a try / catch block. If anything goes wrong,
    // catch the error.
    try {
      // Ensure that the camera is initialized.
      await _initializeControllerFuture;

      // Attempt to take a picture and get the file `image`
      // where it was saved.
      XFile _xFile = await _controller.takePicture();
      _xFileResult = _xFile;
      _cameraSink.add(_xFile.path);
    } catch (e) {
      // If an error occurs, log the error to the console.
      // ignore: avoid_print
      print(e);
    }
  }

  final StreamController<String> _cameraStreamController =
      StreamController<String>.broadcast();
  Sink<String> get _cameraSink => _cameraStreamController.sink;
  Stream<String> get _cameraStream => _cameraStreamController.stream;

  void _closeStream() {
    _cameraStreamController.close();
  }
}
